//**************************************************************************//
//* FRIDA: fast reliable interactive data analysis                         *//
//* coord.cpp: coordinate names and units                                  *//
//* (C) Joachim Wuttke 1990-, v2(C++) 2001-                                *//
//* http://frida.sourceforge.net                                           *//
//**************************************************************************//

#include <math.h>
#include <stdlib.h>
#include <string.h>
using namespace std;

#include "mystd.h"
#include "coord.h"
#include "gar.h"

//**************************************************************************//
//  class CCoord                                                            //
//**************************************************************************//

char CoordDecode( const string& resp, CCoord *rval )
{
    if (resp[0]=='\0') {
        return 'd';
    } else if ( resp=="?" ) {
        return 'h';
    } else if ( resp=="\\EOL" ) {
        *rval = CCoord();
        return 'v';
    }

    uint i, j, np, n;
    n = resp.size();
    if (resp[n-1]!=')') {
        return 'h';
    }
    for(j=n-2, np=1; j!=(uint)-1; j--) {
        if (resp[j]==')') 
            np++;
        else if (resp[j]=='(') {
            np--;
            if (np==0) break;
        }
    }
    if (np!=0) return 'h';

    for (i=j; i>0 && strchr(" \t",resp[i-1]); i--);
    if (i==0) return 'h';
	
    *rval = CCoord( resp.substr(0,i), resp.substr(j+1, n-j-2) );
    return 'v';
}

const char* CCoord::helptext = 
"a coordinate: <name> (<unit>)";

CCoord CCoord::Ask( const string& quest, const CCoord& defval )
{
    *this =  myask( quest, 1, defval, helptext, CoordDecode );
    return *this;
}

CCoord CCoord::Ask( const string& quest )
{
    *this =  myask( quest, 0, CCoord(), helptext, CoordDecode );
    return *this;
}

string CCoord::str() const
{
    return name + ( unit!="" ? (" (" + unit + ")") : "");
}

string CCoord::ps_str() const // just two blanks instead of one
{
    return name + ( unit!="" ? ("  (" + unit + ")") : "");
}

//! Format for saving in a01 format.

string CCoord::to_a01() const
{
    return name + "\t" + unit;
}

string CCoord::load_a01(string in)
{
    string::size_type t1, t2;
    if(!in.size()) {
        name = "";
        unit = "";
        return "";
    }
    if ((t1 = in.find('\t'))==string::npos) {
        name = in;
        unit = "";
        return "";
    }
    name = in.substr(0,t1);
    if ((t2 = in.find('\t', t1+1))==string::npos) {
        unit = in.substr(t1+1);
        return "";
    }
    unit = in.substr(t1+1,t2-t1);
    return in.substr(t2+1);
}

ostream& operator<< (ostream &s, CCoord C)
{
    return s << C.str();
}

//**************************************************************************//
//  class CParam                                                            //
//**************************************************************************//

string CParam::str() const
{
    return strg(val) + string(" ") + Co.str();
}
